//###########################################################################
// FILE:	CCANDataLinkLayer.c
// TITLE:	The CAN Data Link Layer Support Functions.
//###########################################################################
#include "CBaseSubSystem.h"
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Semaphore.h>
//#include <ti/sysbios/gates/GateTask.h>
#include "CCANDataLinkLayer.h"
#include "CBaseSubSystem.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <xdc/cfg/global.h>

#include <xdc/runtime/Log.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Log.h>



#include <xdc/cfg/global.h>


#include "ProjectConfiguration.h"

// Initialize the instance pointer
CCANDataLinkLayer* CCANDataLinkLayer::m_pCanDataLinkPtr = NULL;

extern Int HeapSegment;

//////////////////////////////////////////////////////////////////////
//        ****** CCANDataLinkLayer Functions ******
//////////////////////////////////////////////////////////////////////


/*******************************************************************************
Function Name : CCANDataLinkLayer
Description :  	This is the Constructor function.
*******************************************************************************/
CCANDataLinkLayer::CCANDataLinkLayer()
{
	memset(SubSystemPtrArr,0,sizeof(SubSystemPtrArr));
	memset(cACKReceivedDataArr,0,sizeof(char)*CAN_MAX_DATA_SIZE);
	uiACKReceivedDataSize = 0;
	cNACKerrorCode = 0;
	m_bAckReceived = FALSE;
	m_bNackReceived = FALSE;
	GateTask_Params_init(&prms);

}

/*******************************************************************************
Function Name : ~CCANDataLinkLayer
Description :  	This is the Destructor function.
*******************************************************************************/
CCANDataLinkLayer::~CCANDataLinkLayer()
{
}

/*******************************************************************************
Function Name : Init
Description :  	This is the Init function.
*******************************************************************************/
void CCANDataLinkLayer::Init(CAN_INTERFACE_NUM CanBusSelect)
{ 
    Log_info0("CCANDataLinkLayer::Init start ");

	m_pCanPhysicalPtr = (CCANPhysicalLayer *)CCANPhysicalLayer::Getinstance(); 

	if ( m_pCanPhysicalPtr != NULL)
	{
		m_pCanPhysicalPtr->HWconfig(CanBusSelect);
		m_pCanPhysicalPtr->Init();
	}
	else 
	{
	    Log_info0("CCANDataLinkLayer::Init error ");
	// Error report
	}
    Log_info0("CCANDataLinkLayer::Init end ");
}

/*******************************************************************************
Function Name : Init
Description :  	This is the Init function.
*******************************************************************************/
STATUS CCANDataLinkLayer::SetBoard_Id(Uint16 uiBrd_Id)
{
    Log_info0("CCANDataLinkLayer::SetBoard_Id start ");
	Uint16 uiRet = NO_ERROR;

	// 0xff board id is reserved for monitor PC
	if(uiBrd_Id < TRACE_BOARD_ID)
	{
		m_pCanPhysicalPtr->uiBoard_Id = uiBrd_Id;
	}
	else 
	{
		uiRet = CAN_DATA_LINK_BOARD_ID_SETTING_ERROR; 
	}

	return uiRet;
}


/*******************************************************************************
Function Name : TransmitMsg
Description :  	This function transmit CAN message.
*******************************************************************************/
STATUS CCANDataLinkLayer::TransmitMsg(CAN_MESSAGE * CanTxPacket)
{
	Uint16 m_iStatus = NO_ERROR;

	/*UInt gateKey;
	GateTask_Handle gateTask;
	GateTask_Params prms;
	GateTask_Params_init(&prms);
*/

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);

	  // Simultaneous operations on a List by multiple threads could
	  // corrupt the List structure, so modifications to the List
	  // are protected with a Gate.


/*	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR; 
		return m_iStatus;
	}*/

	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(CanTxPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	return m_iStatus;
}

/*******************************************************************************
Function Name : CANSendBROADCASTmsg
Description :  	This function transmit BROADCAST message.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendBROADCASTmsg( char cVirtualRegID, char cDataSize, char *pcDataBuff)
{
	Uint16 m_iStatus = NO_ERROR;
	CAN_MESSAGE CanPacket;
	char cSendDataArr[CAN_MAX_DATA_SIZE];

	if ( cDataSize > CAN_MAX_DATA_SIZE )
	{
		m_iStatus = CAN_DATA_LINK_DATA_PACKET_ERROR;
		return m_iStatus;
	}

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);

/*	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	memset(cSendDataArr,0,sizeof(char)*CAN_MAX_DATA_SIZE);
	memcpy(cSendDataArr, pcDataBuff,cDataSize );

	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_BROADCAST;
	CanPacket.MsgId.bit.BoardID = BROADCAST_BOARD_ID; // cBoardId is Dont Care !
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	// cSubsystemID is Dont Care, always handled by board SubSystem !
	CanPacket.MsgId.all |= (((Uint32)SUBSYSTEM_TYPE_BOARD)<<13); 
	CanPacket.uiPacketDataArrSize = cDataSize;
	CanPacket.uiPacketDataL = (cSendDataArr[0]&0xFF)+((cSendDataArr[1]<<8)&0xff00)+(((Uint32)cSendDataArr[2]<<16)&0xff0000)+(((Uint32)cSendDataArr[3]<<24)&0xff000000);
	CanPacket.uiPacketDataH = (cSendDataArr[4]&0xFF)+((cSendDataArr[5]<<8)&0xff00)+(((Uint32)cSendDataArr[6]<<16)&0xff0000)+(((Uint32)cSendDataArr[7]<<24)&0xff000000);

	// in Broadcast Msg do not wait for HW ack, so there is no need to check in transmission successd.
	m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);


	return m_iStatus;
}

/*******************************************************************************
Function Name : CANSendImmediateMsg
Description :  	This function transmit Immediate message.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendIMMEDIATEmsg(char cBoardId,char cSubsystemID,char cCommand)
{
	CAN_MESSAGE CanPacket;
	Uint16 m_iStatus = NO_ERROR;

	if ( cBoardId > 0xFF )
	{
		m_iStatus = CAN_DATA_LINK_DATA_PACKET_ERROR;
		return m_iStatus;
	}

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);



/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_IMMEDIATE;
	CanPacket.MsgId.bit.BoardID = cBoardId;
	CanPacket.MsgId.bit.VirtualRegID = cCommand;
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); 
	CanPacket.uiPacketDataArrSize = 0;
	CanPacket.uiPacketDataL = 0;
	CanPacket.uiPacketDataH = 0;
 	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	return m_iStatus;
}

/*******************************************************************************
Function Name : CANSendUNSOLICITEDmsg
Description :  	This function transmit UNSOLICITED message.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendUNSOLICITEDmsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char cDataSize, char *pcDataBuff)
{
	Uint16 m_iStatus = NO_ERROR;
	CAN_MESSAGE CanPacket;
	char cSendDataArr[CAN_MAX_DATA_SIZE];


	if ( (cDataSize > CAN_MAX_DATA_SIZE)||(cBoardId > 0xFF) )
	{
		m_iStatus = CAN_DATA_LINK_DATA_PACKET_ERROR;
		return m_iStatus;
	}

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);



/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	memset(cSendDataArr,0,sizeof(char)*CAN_MAX_DATA_SIZE);
	memcpy(cSendDataArr, pcDataBuff,cDataSize );

	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_UNSOLICITED_DATA;
	CanPacket.MsgId.bit.BoardID = cBoardId;
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); // New
	CanPacket.uiPacketDataArrSize = cDataSize;
	CanPacket.uiPacketDataL = (cSendDataArr[0]&0xFF)+((cSendDataArr[1]<<8)&0xff00)+(((Uint32)cSendDataArr[2]<<16)&0xff0000)+(((Uint32)cSendDataArr[3]<<24)&0xff000000);
	CanPacket.uiPacketDataH = (cSendDataArr[4]&0xFF)+((cSendDataArr[5]<<8)&0xff00)+(((Uint32)cSendDataArr[6]<<16)&0xff0000)+(((Uint32)cSendDataArr[7]<<24)&0xff000000);
	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	
	return m_iStatus;
}

/*******************************************************************************
Function Name : SendUNSOLICITEDbuffer
Description :  	This function
*******************************************************************************/
STATUS CCANDataLinkLayer::SendUNSOLICITEDbuffer(char cPacketID,char cBoardId,char cSubsystemID, Uint16 uiDataSize, char *pcDataBuff)
{
	Uint16 uiBytesToTransmit;
	Uint16 uiNumTxPacket;
	Uint16 i;
	Uint16 uiPacketNum;
	Uint16 m_iStatus = NO_ERROR;
	CAN_MESSAGE CanPacket;
	

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);



/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	CanPacket.MsgId.bit.BoardID = cBoardId; 
	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_UNSOLICITED_BUFFER_DATA;//CAN_ID_MSG_TYPE_UNSOLICITED_DATA;
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); // New

	if ( uiDataSize <= CAN_MAX_DATA_SIZE ) 
	{
		CanPacket.uiPacketDataArrSize = uiDataSize;
		CanPacket.uiPacketDataL = (pcDataBuff[0]&0xFF)+((pcDataBuff[1]<<8)&0xff00)+(((Uint32)pcDataBuff[2]<<16)&0xff0000)+(((Uint32)pcDataBuff[3]<<24)&0xff000000);
		CanPacket.uiPacketDataH = (pcDataBuff[4]&0xFF)+((pcDataBuff[5]<<8)&0xff00)+(((Uint32)pcDataBuff[6]<<16)&0xff0000)+(((Uint32)pcDataBuff[7]<<24)&0xff000000);
		CanPacket.MsgId.bit.VirtualRegID = (0x01<<2)+ONE_PACKET_BUFFER;
		m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	}
	else if ( uiDataSize > CAN_MAX_DATA_SIZE )
	{
		uiBytesToTransmit = uiDataSize;
		uiNumTxPacket = ceil((double)uiDataSize/CAN_MAX_DATA_SIZE);
		CanPacket.MsgId.bit.VirtualRegID = (uiNumTxPacket<<2) + START_BUFFER_FIRST_PACKET;
		uiPacketNum = 1;

		for( i=0; i<uiNumTxPacket; i++ )
		{
			if ( uiBytesToTransmit <= CAN_MAX_DATA_SIZE ) 
			{
				CanPacket.uiPacketDataArrSize = uiBytesToTransmit;
				CanPacket.MsgId.bit.VirtualRegID = (uiPacketNum<<2)+END_BUFFER_LAST_PACKET;
			}
			else 
			{
				if ( i > 0 )
				{
					CanPacket.MsgId.bit.VirtualRegID = (uiPacketNum<<2) + BUFFER_MIDDLE_PACKET;
				}
				CanPacket.uiPacketDataArrSize = CAN_MAX_DATA_SIZE;
				uiBytesToTransmit -=CAN_MAX_DATA_SIZE;
			}
			CanPacket.uiPacketDataL = (pcDataBuff[8*i]&0xFF)+((pcDataBuff[(8*i)+1]<<8)&0xff00)+(((Uint32)pcDataBuff[(8*i)+2]<<16)&0xff0000)+(((Uint32)pcDataBuff[(8*i)+3]<<24)&0xff000000);
			CanPacket.uiPacketDataH = (pcDataBuff[(8*i)+4]&0xFF)+((pcDataBuff[(8*i)+5]<<8)&0xff00)+(((Uint32)pcDataBuff[(8*i)+6]<<16)&0xff0000)+(((Uint32)pcDataBuff[(8*i)+7]<<24)&0xff000000);
			m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
			if ( m_iStatus != NO_ERROR)
			{
				return m_iStatus;
			}
			uiPacketNum++;
		}
	}

	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	
	return m_iStatus;
}

/*******************************************************************************
Function Name : CANSendAck
Description :  	This function transmit a packet Acknowledge with or without the data.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendAppAck( char cSubsystemID,char cVirtualRegID,Uint16 cDataSize, char *pcDataBuff)
{
	CAN_MESSAGE CanPacket;
	Uint16 m_iStatus;
	char cSendDataArr[CAN_MAX_DATA_SIZE];
	
	if ( cDataSize > CAN_MAX_DATA_SIZE )
	{
		m_iStatus = CAN_DATA_LINK_DATA_PACKET_ERROR;
		return m_iStatus;
	}

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);



/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	memset(cSendDataArr,0,sizeof(char)*CAN_MAX_DATA_SIZE);
	memcpy(cSendDataArr, pcDataBuff,cDataSize );

	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_ACK_RESPONSE;
	CanPacket.MsgId.bit.BoardID = MASTER_BOARD_ID;
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); 
	CanPacket.uiPacketDataArrSize = cDataSize;
	CanPacket.uiPacketDataL = (cSendDataArr[0]&0xFF)+((cSendDataArr[1]<<8)&0xff00)+(((Uint32)cSendDataArr[2]<<16)&0xff0000)+(((Uint32)cSendDataArr[3]<<24)&0xff000000);
	CanPacket.uiPacketDataH = (cSendDataArr[4]&0xFF)+((cSendDataArr[5]<<8)&0xff00)+(((Uint32)cSendDataArr[6]<<16)&0xff0000)+(((Uint32)cSendDataArr[7]<<24)&0xff000000);
	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	return m_iStatus;
}

/*******************************************************************************
Function Name : CANSendNak
Description :  	This function transmit a packet Not Acknowledge with error code
*******************************************************************************/
STATUS CCANDataLinkLayer::SendAppNak(char cSubsystemID,char cVirtualRegID,Uint16 uiErrorCode)
{
	CAN_MESSAGE CanPacket;
	Uint16 m_iStatus;

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);




/*	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_NAK_RESPONSE;
	CanPacket.MsgId.bit.BoardID = MASTER_BOARD_ID;
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); 
	CanPacket.uiPacketDataArrSize = 1;
	CanPacket.uiPacketDataL = uiErrorCode;
	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);
	
	return m_iStatus;
}

/*******************************************************************************
Function Name : SendReadMsg
Description :  	This function transmit read request message.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendReadMsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char *pcRxDataSize, char *pcRxDataBuff, Uint16 uiTimeout)
{
	CAN_MESSAGE CanPacket;
	Uint16 m_iStatus = NO_ERROR;
	Bool bSemRes;

	if ( uiBoard_Id != MASTER_BOARD_ID)
	{
		m_iStatus = CAN_DATA_LINK_BOARD_ID_PERMISSION_ERROR;
		return m_iStatus;
	}

	gateTask = GateTask_create(&prms, NULL);

	if (gateTask == NULL)
	{
	   //System_abort("Gate create failed");
	   m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
	   return m_iStatus;
	}
	  gateKey = GateTask_enter(gateTask);


/*	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_READ_VIRTUAL_REG;
	CanPacket.MsgId.bit.BoardID = cBoardId;
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); 
	CanPacket.uiPacketDataArrSize = 0;
	CanPacket.uiPacketDataL = 0;
	CanPacket.uiPacketDataH = 0;
	m_bNackReceived = FALSE;
	m_bAckReceived = FALSE;
	Semaphore_reset(SEM_CAN_ACK_NAK_RECEIVED, 0);// Reset the 'Ack/Nak received' semaphore
	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);

	bSemRes = Semaphore_pend(SEM_CAN_ACK_NAK_RECEIVED, uiTimeout);
	if (bSemRes == FALSE)	
	{
		m_iStatus = CAN_DATA_LINK_ACK_NAK_TIMEOUT_ERROR;
	}
	else
	{
		if (m_bAckReceived == TRUE)
		{
			// Set returnes data into pcDataBuff;
			pcRxDataBuff = cACKReceivedDataArr;
			*pcRxDataSize = uiACKReceivedDataSize;
			m_iStatus = NO_ERROR;

		}
		else if( m_bNackReceived )
		{
			// Return Nack error code
			pcRxDataBuff[0] = cNACKerrorCode;
			*pcRxDataSize = 1; // One byte error code
			m_iStatus = CAN_DATA_LINK_NAK_RECEIVED_ERROR;
		}
	}

	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	return m_iStatus;
}

/*******************************************************************************
Function Name : SendWriteMsg
Description :  	This function transmit write request message.
*******************************************************************************/
STATUS CCANDataLinkLayer::SendWriteMsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char cDataSize,char *pcRxDataSize, char *pcRxDataBuff, Uint16 uiTimeout)
{
	CAN_MESSAGE CanPacket;
	Uint16 m_iStatus = NO_ERROR;
	Bool bSemRes;
	char cSendDataArr[CAN_MAX_DATA_SIZE];

	if ( uiBoard_Id != MASTER_BOARD_ID)
	{
		m_iStatus = CAN_DATA_LINK_BOARD_ID_PERMISSION_ERROR;
		return m_iStatus;
	}

/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	memset(cSendDataArr,0,sizeof(char)*CAN_MAX_DATA_SIZE);
	memcpy(cSendDataArr, pcRxDataBuff,cDataSize );

	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_WRITE_VIRTUAL_REG;
	CanPacket.MsgId.bit.BoardID = cBoardId;
	CanPacket.MsgId.bit.VirtualRegID = cVirtualRegID; 
	CanPacket.MsgId.all |= (((Uint32)cSubsystemID)<<13); 
	CanPacket.uiPacketDataArrSize = cDataSize;
	CanPacket.uiPacketDataL = (cSendDataArr[0]&0xFF)+((cSendDataArr[1]<<8)&0xff00)+(((Uint32)cSendDataArr[2]<<16)&0xff0000)+(((Uint32)cSendDataArr[3]<<24)&0xff000000);
	CanPacket.uiPacketDataH = (cSendDataArr[4]&0xFF)+((cSendDataArr[5]<<8)&0xff00)+(((Uint32)cSendDataArr[6]<<16)&0xff0000)+(((Uint32)cSendDataArr[7]<<24)&0xff000000);

	m_bNackReceived = FALSE;
	m_bAckReceived = FALSE;
	Semaphore_reset(SEM_CAN_ACK_NAK_RECEIVED, 0);// Reset the 'Ack/Nak received' semaphore
	m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);

	bSemRes = Semaphore_pend(SEM_CAN_ACK_NAK_RECEIVED, uiTimeout);
	if (bSemRes == FALSE)
	{
		m_iStatus = CAN_DATA_LINK_ACK_NAK_TIMEOUT_ERROR;
	}
	else
	{
		if (m_bAckReceived == TRUE)
		{
			// Set returnes data into pcDataBuff;
			pcRxDataBuff = cACKReceivedDataArr;
			*pcRxDataSize = uiACKReceivedDataSize;
			m_iStatus = NO_ERROR;
		}
		else if( m_bNackReceived )
		{
			// Return Nack error code
			pcRxDataBuff[0] = cNACKerrorCode;
			*pcRxDataSize = 1; // One byte error code
			m_iStatus = CAN_DATA_LINK_NAK_RECEIVED_ERROR;
		}
	}

	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	return m_iStatus;
}

/*******************************************************************************
Function Name : SendWriteMsg
Description :  	This function transmit write request message.
*******************************************************************************/
STATUS CCANDataLinkLayer::Register( Uint16 uiSubSystemID,void * p_MyClass)
{
    Log_info0("CCANDataLinkLayer::Register start ");

	if (  uiSubSystemID < MAX_SUBSYSTEMS )
		SubSystemPtrArr[uiSubSystemID] = p_MyClass;
	else 
		return CAN_DATA_LINK_SS_REGISTER_ERROR;

	return NO_ERROR;
}

/*******************************************************************************
Function Name : CANhandleReceivePacket
Description :  	This function is called every time a full packet is received.
*******************************************************************************/
void CCANDataLinkLayer::HandleReceivePacket(void)
{

    Log_info0("CCANDataLinkLayer::HandleReceivePacket start ");

	int16 iRet = NO_ERROR;
	static CAN_MESSAGE CanRxData;
//	static char cReadDataArr[CAN_MAX_DATA_SIZE];
	char * cReadDataArr;
	static Uint16 uiReadDataSize = 0;
	static Uint16 uiSSid = 0;
	static Uint16 j=0,i=0;
	static Uint16 uiNumOfPackets = 0;
	static Uint16 uiBuffNum = 0;
	static Uint16 uiTotalBytesNum = 0;
	static Uint16 uiCommand = 0;

	//MEM_Stat statbuf;
	//Bool status = MEM_stat(HeapSegment, &statbuf);

   Log_info0("CCANDataLinkLayer::before while ");

	while (1)
	{
		// Wait for a message to be recieved
		//if (SEM_pend(&SEM_CAN_MSG_RECEIVED, SYS_FOREVER)) //SEM_CAN_RECEIVE

		if (Semaphore_pend(SEM_CAN_BROADCAST_MSG_RECEIVED, BIOS_WAIT_FOREVER)) //SEM_CAN_BROADCAST_MSG_RECEIVED
		{
			   Log_info0("SEM_CAN_BROADCAST_MSG_RECEIVED  ");
				if( m_pCanPhysicalPtr->ReceiveFifoMsg(&CanRxData) == CAN_NO_MESSAGE_WAIT_ERROR)
				{
					 Log_info0("CAN_NO_MESSAGE_WAIT_ERROR  ");

					//continue;
				}

				Log_info0("Message received  ");
				uiSSid = (CanRxData.MsgId.bit.FCopmSSID_L)+(CanRxData.MsgId.bit.FCopmSSID_H<<3);
				Log_info1("CCANDataLinkLayer/MsgType uiSSid = %d",uiSSid);
				CAN_MESSAGE CanData;
				                CanData.uiPacketDataArrSize =1;
				                CanData.MsgId.all =0x80050000;
				                CanData.uiPacketDataH =0x123;
				                CanData.uiPacketDataL =((Uint32)123);
				m_pCanPhysicalPtr->TransmitMsg(&CanData,CAN_ID_PRIORITY_LEVEL_NORMAL);
				Log_info0("Message sent  ");
				continue;
		}

		if (Semaphore_pend(SEM_CAN_MSG_RECEIVED, BIOS_WAIT_FOREVER)) //SEM_CAN_RECEIVE
		{

		   Log_info0("SEM_CAN_MSG_RECEIVED  ");

			if( m_pCanPhysicalPtr->ReceiveFifoMsg(&CanRxData) == CAN_NO_MESSAGE_WAIT_ERROR)
			{
				 Log_info0("CAN_NO_MESSAGE_WAIT_ERROR  ");

				continue;
			}

			 Log_info0("Message received  ");

			uiSSid = (CanRxData.MsgId.bit.FCopmSSID_L)+(CanRxData.MsgId.bit.FCopmSSID_H<<3);

			Log_info1("CCANDataLinkLayer/MsgType uiSSid = %d",uiSSid);

			switch (CanRxData.MsgId.bit.MsgType)
			{
				case CAN_ID_MSG_TYPE_IMMEDIATE:
					if (SubSystemPtrArr[uiSSid] != NULL)
						((CBaseSubSystem *)SubSystemPtrArr[uiSSid])->MyExecuteCANCommand(CanRxData.MsgId.bit.VirtualRegID,0,NULL,0);
				break;

				case CAN_ID_MSG_TYPE_UNSOLICITED_DATA:
					cReadDataArr = new char[CAN_MAX_DATA_SIZE];
					if (cReadDataArr == 0)
					{
						//LOG_printf(&trace, "Packet 'data' memory allocation failed");
					}
					if (cReadDataArr != 0)
					{
						for(i=0; i<4; i++)
						{
							cReadDataArr[i] = (CanRxData.uiPacketDataL>>8*i)&0xff;
							cReadDataArr[4+i] = (CanRxData.uiPacketDataH>>8*i)&0xff;
						}
						if (SubSystemPtrArr[uiSSid] != NULL)
							((CBaseSubSystem *)SubSystemPtrArr[uiSSid])->MyExecuteCANCommand(CanRxData.MsgId.bit.VirtualRegID,CanRxData.uiPacketDataArrSize,cReadDataArr,0);
						// Free the message allocated memory
						delete []cReadDataArr;
						cReadDataArr = NULL;
					}
					break;

				case CAN_ID_MSG_TYPE_UNSOLICITED_BUFFER_DATA :
					uiNumOfPackets = (CanRxData.MsgId.bit.VirtualRegID>>2);
					if( ( (CanRxData.MsgId.bit.VirtualRegID & 0x03) == START_BUFFER_FIRST_PACKET ) ||
						((CanRxData.MsgId.bit.VirtualRegID & 0x03) == ONE_PACKET_BUFFER ) )
					{
						cReadDataArr = new char[CAN_MAX_DATA_SIZE*uiNumOfPackets];
						uiBuffNum = 1;
						i = 0;
						if (cReadDataArr == 0)
						{
							//LOG_printf(&trace, "Packet 'data' memory allocation failed");
						}
						if (cReadDataArr != 0)
						{
							uiTotalBytesNum = CanRxData.uiPacketDataArrSize;
							for(i=0; i<4; i++)
							{
								if ( i == 1)
									uiCommand = (CanRxData.uiPacketDataL>>8*i)&0xff;
								cReadDataArr[i] = (CanRxData.uiPacketDataL>>8*i)&0xff;
								cReadDataArr[4+i] = (CanRxData.uiPacketDataH>>8*i)&0xff;
							}
						}
					}
					else //  Middle or end of packet
					{
						j = 8 *uiBuffNum;
						uiBuffNum++;
						uiTotalBytesNum += CanRxData.uiPacketDataArrSize;
						for(i=0; i<4; i++,j++)
						{
							cReadDataArr[j] = (CanRxData.uiPacketDataL>>8*i)&0xff;
							cReadDataArr[4+j] = (CanRxData.uiPacketDataH>>8*i)&0xff;
						}
					}

					if( ( (CanRxData.MsgId.bit.VirtualRegID & 0x03) == END_BUFFER_LAST_PACKET )
						|| ((CanRxData.MsgId.bit.VirtualRegID & 0x03) == ONE_PACKET_BUFFER ) )
					{
						if ( uiBuffNum == uiNumOfPackets)
						{
							cReadDataArr[1] = cReadDataArr[0];// Remove command from buffer - replace with 
							if (SubSystemPtrArr[uiSSid] != NULL)
								((CBaseSubSystem *)SubSystemPtrArr[uiSSid])->MyExecuteCANCommand(uiCommand,uiTotalBytesNum,&cReadDataArr[1],0);
							uiTotalBytesNum = 0;
						}
						// else Error receiving packet

						// Free the message allocated memory
						if (cReadDataArr != 0)
						{
							delete []cReadDataArr;
							cReadDataArr = NULL;
						}
					}
					break;
				case CAN_ID_MSG_TYPE_ACK_RESPONSE:
					// Pass returned read data to the requester 
					// Merav - check  the received ss if the equal to requester
					for(i=0; i<4; i++)
					{
						cACKReceivedDataArr[i] = (CanRxData.uiPacketDataL>>8*i)&0xff;
						cACKReceivedDataArr[4+i] = (CanRxData.uiPacketDataH>>8*i)&0xff;
					}
					uiACKReceivedDataSize = CanRxData.uiPacketDataArrSize;
					m_bAckReceived = TRUE;
					//SEM_post(&SEM_CAN_ACK_NAK_RECEIVED);
					Semaphore_post(SEM_CAN_ACK_NAK_RECEIVED);
					break;

				case CAN_ID_MSG_TYPE_NAK_RESPONSE:
					cNACKerrorCode = (char)(CanRxData.uiPacketDataL&0xff);
					m_bNackReceived = TRUE;
					//SEM_post(&SEM_CAN_ACK_NAK_RECEIVED);
					Semaphore_post(SEM_CAN_ACK_NAK_RECEIVED);
					break;

				case CAN_ID_MSG_TYPE_WRITE_VIRTUAL_REG:
					for(i=0; i<4; i++)
					{    
						cReadDataArr[i] = (CanRxData.uiPacketDataL>>8*i)&0xff;
						cReadDataArr[4+i] = (CanRxData.uiPacketDataH>>8*i)&0xff;
					}
					if (SubSystemPtrArr[uiSSid] != NULL)
					{
						iRet = ((CBaseSubSystem *)SubSystemPtrArr[uiSSid])->MyExecuteCANCommand(CanRxData.MsgId.bit.VirtualRegID,CanRxData.uiPacketDataArrSize,cReadDataArr,&uiReadDataSize);
					}
					else 
						iRet = CAN_DATA_LINK_DATA_PACKET_ERROR;
					if ( iRet == NO_ERROR)
					{
						// ACK Response without data
						SendAppAck(uiSSid,CanRxData.MsgId.bit.VirtualRegID,0,NULL);
					}
					else 
					{
						SendAppNak(uiSSid,CanRxData.MsgId.bit.VirtualRegID,iRet);
					}
					break;

				case CAN_ID_MSG_TYPE_READ_VIRTUAL_REG:
					if (SubSystemPtrArr[uiSSid] != NULL)
					{
						iRet = ((CBaseSubSystem *)SubSystemPtrArr[uiSSid])->MyExecuteCANCommand(CanRxData.MsgId.bit.VirtualRegID,0,cReadDataArr,&uiReadDataSize);
					}  
					else 
						iRet = CAN_DATA_LINK_DATA_PACKET_ERROR;
					if ( iRet == NO_ERROR)
					{
						// ACK Response without data
						SendAppAck(uiSSid,CanRxData.MsgId.bit.VirtualRegID,uiReadDataSize,cReadDataArr);
					}
					else 
					{
						SendAppNak(uiSSid,CanRxData.MsgId.bit.VirtualRegID,iRet);
					}
					break;
				default:
					Log_info0("CCANDataLinkLayer/default");

					SendAppAck(uiSSid,CanRxData.MsgId.bit.VirtualRegID,0,NULL);
					break;

			}
		}
	}

}


/*******************************************************************************
Function Name : SendTraceMsg
Description :  	This function
*******************************************************************************/
STATUS CCANDataLinkLayer::SendTraceMsg(char cBuffNum,Uint16 uiDataSize, char *pcDataBuff)
{
	Uint16 uiBytesToTransmit;
	Uint16 uiNumTxPacket;
	Uint16 i;
	Uint16 uiPacketNum;
	Uint16 m_iStatus = NO_ERROR;
	CAN_MESSAGE CanPacket;
	
	//char cSendDataArr[CAN_MAX_DATA_SIZE];

	if ( uiBoard_Id != MASTER_BOARD_ID)
	{
		m_iStatus = CAN_DATA_LINK_BOARD_ID_PERMISSION_ERROR;
		return m_iStatus;
	}

/*
	bStatus = LCK_pend(&LCK_CAN_Transmit,CAN_LCK_TRANSMIT_TIMEOUT);
	if ( bStatus == false)
	{
		m_iStatus = CAN_DATA_LINK_LCK_TRANSMITTER_ERROR;
		return m_iStatus;
	}
*/

	memset(&CanPacket,0, sizeof(CAN_MESSAGE));
	CanPacket.MsgId.bit.BoardID = TRACE_BOARD_ID;
	CanPacket.MsgId.bit.PriorityLevel = CAN_ID_PRIORITY_LEVEL_NORMAL;
	CanPacket.MsgId.bit.MsgType = CAN_ID_MSG_TYPE_UNSOLICITED_DATA;
	CanPacket.MsgId.bit.FCopmSSID_L = 7; // 11111 for general buffer use !
	CanPacket.MsgId.bit.FCopmSSID_H = (cBuffNum<<2)+3;

	if ( uiDataSize <= CAN_MAX_DATA_SIZE ) 
	{
		CanPacket.uiPacketDataArrSize = uiDataSize;
		CanPacket.uiPacketDataL = (pcDataBuff[0]&0xFF)+((pcDataBuff[1]<<8)&0xff00)+(((Uint32)pcDataBuff[2]<<16)&0xff0000)+(((Uint32)pcDataBuff[3]<<24)&0xff000000);
		CanPacket.uiPacketDataH = (pcDataBuff[4]&0xFF)+((pcDataBuff[5]<<8)&0xff00)+(((Uint32)pcDataBuff[6]<<16)&0xff0000)+(((Uint32)pcDataBuff[7]<<24)&0xff000000);
		CanPacket.MsgId.bit.VirtualRegID = ONE_PACKET_BUFFER;
		m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
	}
	else if ( uiDataSize > CAN_MAX_DATA_SIZE )
	{
		uiBytesToTransmit = uiDataSize;
		uiNumTxPacket = ceil((double)uiDataSize/CAN_MAX_DATA_SIZE);
		CanPacket.MsgId.bit.VirtualRegID = (uiNumTxPacket<<2) + START_BUFFER_FIRST_PACKET;
		uiPacketNum = 1;

		for( i=0; i<uiNumTxPacket; i++ )
		{
			if ( uiBytesToTransmit <= CAN_MAX_DATA_SIZE ) 
			{
				CanPacket.uiPacketDataArrSize = uiBytesToTransmit;
				CanPacket.MsgId.bit.VirtualRegID = (uiPacketNum<<2)+END_BUFFER_LAST_PACKET;
			}
			else 
			{
				if ( i > 0 )
				{
					CanPacket.MsgId.bit.VirtualRegID = (uiPacketNum<<2) + BUFFER_MIDDLE_PACKET;
				}
				CanPacket.uiPacketDataArrSize = CAN_MAX_DATA_SIZE;
				uiBytesToTransmit -=CAN_MAX_DATA_SIZE;
			}
			CanPacket.uiPacketDataL = (pcDataBuff[8*i]&0xFF)+((pcDataBuff[(8*i)+1]<<8)&0xff00)+(((Uint32)pcDataBuff[(8*i)+2]<<16)&0xff0000)+(((Uint32)pcDataBuff[(8*i)+3]<<24)&0xff000000);
			CanPacket.uiPacketDataH = (pcDataBuff[(8*i)+4]&0xFF)+((pcDataBuff[(8*i)+5]<<8)&0xff00)+(((Uint32)pcDataBuff[(8*i)+6]<<16)&0xff0000)+(((Uint32)pcDataBuff[(8*i)+7]<<24)&0xff000000);
			m_iStatus = m_pCanPhysicalPtr->TransmitMsg(&CanPacket,CAN_ID_PRIORITY_LEVEL_NORMAL);
			if ( m_iStatus != NO_ERROR)
			{
				return m_iStatus;
			}
			uiPacketNum++;
		}
	}

	//LCK_post(&LCK_CAN_Transmit);
	GateTask_leave(gateTask, gateKey);

	
	return m_iStatus;
}


