//###########################################################################
// FILE:	CCANDataLinkLayer.h
// TITLE:	This file include the CAN protocol .
//###########################################################################
#ifndef CCANDataLinkLayer_H
#define CCANDataLinkLayer_H
#include <ti/sysbios/gates/GateTask.h>

#include "CCANPhysicalLayer.h"
//#include "CGenericBoardSubSystem.h"

#ifdef __cplusplus
extern "C" {
#endif

//////////////////////////////////////////////////////////////////////
//  ****** CCANDataLinkLayer Class ******
//////////////////////////////////////////////////////////////////////
class CCANDataLinkLayer
{
friend class CBoardSubSystem; 
private:
// methods
	CCANDataLinkLayer();
	~CCANDataLinkLayer();
	STATUS TransmitMsg(CAN_MESSAGE * CanPacket); // Merav - Not used
	STATUS SendAppAck(char cSubsystemID,char cVirtualRegID,Uint16 cDataSize, char *pcDataBuff); // Sends a packet Acknowledge with or without the data to Master
	STATUS SendAppNak(char cSubsystemID,char cVirtualRegID,Uint16 uiErrorCode);// Sends a packet Not Acknowledge with error code to Master
// Data members
	static CCANDataLinkLayer *m_pCanDataLinkPtr;
	void * SubSystemPtrArr[MAX_SUBSYSTEMS];
	char cACKReceivedDataArr[8];
	Uint16 uiACKReceivedDataSize;
	char cNACKerrorCode;

	UInt gateKey;
	GateTask_Handle gateTask;
	GateTask_Params prms;


public:
	void Init(CAN_INTERFACE_NUM CanBusSelect);
	STATUS SendBROADCASTmsg(char cVirtualRegID,char cDataSize, char *pcDataBuff);
	STATUS SendIMMEDIATEmsg(char cBoardId, char cSubsystemID,char cCommand);
	STATUS SendUNSOLICITEDmsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char cDataSize, char *pcDataBuff);
	STATUS SendUNSOLICITEDbuffer(char cPacketID,char cSubsystemID,char cBoardId,Uint16 uiDataSize, char *pcDataBuff);
 	STATUS SendReadMsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char *pcRxDataSize, char *pcRxDataBuff, Uint16 uiTimeout=CAN_ACK_NAK_TIMEOUT);
	STATUS SendWriteMsg(char cBoardId,char cSubsystemID,char cVirtualRegID, char cDataSize,char *pcRxDataSize, char *pcRxDataBuff, Uint16 uiTimeout=CAN_ACK_NAK_TIMEOUT);
	STATUS SendTraceMsg(char cBuffNum,Uint16 uiDataSize, char *pcDataBuff);
	void HandleReceivePacket();		// Handle a recieved packet from any type (data/ack/nak..)
	STATUS Register( Uint16 uiSubSystemID,void * p_MyClass);

	static CCANDataLinkLayer* Getinstance()
	{
		if (NULL == m_pCanDataLinkPtr)
			m_pCanDataLinkPtr = new CCANDataLinkLayer;
		return m_pCanDataLinkPtr; 
	}


	CCANPhysicalLayer * m_pCanPhysicalPtr;
	Uint16 uiBoard_Id;
protected:
	STATUS SetBoard_Id(Uint16 uiBrd_Id);
	Bool m_bAckReceived; 
	Bool m_bNackReceived; 
};



#ifdef __cplusplus
}
#endif /* extern "C" */



#endif 
/*** end of file *****************************************************/



