//###########################################################################
// FILE:	CCANGlobalDef.h
// TITLE:	This file include the CAN Physical Layer .
//###########################################################################
#ifndef CCANGlobalDef_H
#define CCANGlobalDef_H

#include "ProjectConfiguration.h"

#ifdef __cplusplus
extern "C" {
#endif


// Extended CAN Identifier  - 29 bits description
struct  CAN_ID_BITS 
{
   Uint16  PriorityLevel:1; // 0	 Priority Level   
   Uint16  rsvd:1;   		// 1	 reserved   
   Uint16  MsgType:3;    	// 2:4	 Message Type
   Uint16  BoardID:8;     	// 5:12  Board ID
   Uint16  FCopmSSID_L:3;   // 13:20 Functional components / subsystem ID
   Uint16  FCopmSSID_H:5;   	
   Uint16  VirtualRegID:8;	// 21:28 Virtual register ID / Immediate commands
   Uint16  rsvd1:3;   		// 29-31 reserved   
};

// Allow access to the bit fields or entire Identifier
union CAN_HEADER_ID 
{
   Uint32              all;
   struct CAN_ID_BITS  bit;
};

struct CAN_MESSAGE 
{
   	union CAN_HEADER_ID   MsgId;         
	Uint32 uiPacketDataL ;
	Uint32 uiPacketDataH ;
	Uint16 uiPacketDataArrSize;
};

#define CAN_MAX_DATA_SIZE				8  // Max data size is 8 bytes
#define CAN_TRANSMIT_TIMEOUT			10 // 10 milliseconds //10000
#define TRANSMIT_MAILBOX				0  // 0 - TRANSMIT Mailbox
#define RECEIVE_MAILBOX					1  // 1 - RECEIVE Mailbox
// Overwrite protection control bits
#define SET_OVERWRITE_PROTECT_CONTROL	1  // OPC[n] is set to 1
#define RESET_OVERWRITE_PROTECT_CONTROL	0  // OPC[n] is set to 0

#define ENABLE_MAILBOX_INTERRUPT		1  
#define DISABLE_MAILBOX_INTERRUPT		0 

enum {
	MAILBOX_NUM_0 = 0,
	MAILBOX_NUM_1 = 1,
	MAILBOX_NUM_2 = 2, 
	MAILBOX_NUM_3 = 3, 		
	MAILBOX_NUM_4 = 4,	
	MAILBOX_NUM_5 = 5,
	MAILBOX_NUM_6 = 6,
	MAILBOX_NUM_7 = 7,	
	MAILBOX_NUM_8 = 8,
	MAILBOX_NUM_9 = 9,	
	MAILBOX_NUM_10 = 10,
	MAILBOX_NUM_11 = 11, 
	MAILBOX_NUM_12 = 12, 		
	MAILBOX_NUM_13 = 13,	
	MAILBOX_NUM_14 = 14,
	MAILBOX_NUM_15 = 15,
	MAILBOX_NUM_16 = 16,	
	MAILBOX_NUM_17 = 17,
	MAILBOX_NUM_18 = 18,	
	MAILBOX_NUM_19 = 19,
	MAILBOX_NUM_20 = 20, 
	MAILBOX_NUM_21 = 21, 		
	MAILBOX_NUM_22 = 22,	
	MAILBOX_NUM_23 = 23,
	MAILBOX_NUM_24 = 24,
	MAILBOX_NUM_25 = 25,	
	MAILBOX_NUM_26 = 26,
	MAILBOX_NUM_27 = 27,	
	MAILBOX_NUM_28 = 28,	
	MAILBOX_NUM_29 = 29,
	MAILBOX_NUM_30 = 30, 
	MAILBOX_NUM_31 = 31
};

// Define special mailboxes number
#define	IMMEDIATE_RECEIVE_MAILBOX 		MAILBOX_NUM_31
#define	BROADCAST_RECEIVE_MAILBOX 		MAILBOX_NUM_30
#define	CHARGER_RECEIVE_MAILBOX 		MAILBOX_NUM_2

#define CAN_ID_PRIORITY_LEVEL_HIGH 		0
#define CAN_ID_PRIORITY_LEVEL_NORMAL	1

#define MAX_SUBSYSTEMS 		15
#define MASTER_BOARD_ID 	0x00
#define TRACE_BOARD_ID 		0xFF
#define BROADCAST_BOARD_ID 	0xFF // BROADCAST_BOARD_ID is Dont Care !

#define BUFFER_MIDDLE_PACKET			0x00
#define START_BUFFER_FIRST_PACKET 		0x01
#define END_BUFFER_LAST_PACKET			0x02
#define ONE_PACKET_BUFFER				0x03

#define CAN_ACK_NAK_TIMEOUT				100	// 200 mille sec timeout delay for the receiving of an Ack packet
#define CAN_LCK_TRANSMIT_TIMEOUT		500	


enum {
	CAN_ID_MSG_TYPE_BROADCAST = 0,
	CAN_ID_MSG_TYPE_IMMEDIATE,
	CAN_ID_MSG_TYPE_UNSOLICITED_DATA, 	//(no ACK needed)
	CAN_ID_MSG_TYPE_UNSOLICITED_BUFFER_DATA, 	//(buffer more than 8 bytes, no ACK needed)
	CAN_ID_MSG_TYPE_ACK_RESPONSE, 		//(0-8 bytes of data) 
	CAN_ID_MSG_TYPE_NAK_RESPONSE,		//(1 byte of data = error code)
	CAN_ID_MSG_TYPE_WRITE_VIRTUAL_REG,	//(ACK response required)
	CAN_ID_MSG_TYPE_READ_VIRTUAL_REG	//(ACK response required)
};

typedef enum 
{
	CAN_A_INTERFACE = 0,	// eCAN-A module
	CAN_B_INTERFACE = 1		// eCAN-B module
}CAN_INTERFACE_NUM;


//---------------------------------------------------------------------------
#ifdef __cplusplus
}
#endif /* extern "C" */

#endif  

/*** end of file *****************************************************/
