//###########################################################################
// FILE:	CCANPhysicalLayer.c
// TITLE:	This file include the CAN Physical Layer.
//###########################################################################
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <string.h>
#include <xdc/cfg/global.h>

#include <xdc/runtime/Log.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/System.h>
#include "CCANPhysicalLayer.h"
#include <xdc/runtime/Log.h>
// Initialize the instance pointer
CCANPhysicalLayer* CCANPhysicalLayer::m_pCanInstancePtr = NULL;

// Globals
struct ECAN_REGS ECanShadow;
int	MIV = 0; // Stores the mailbox # that needs to be serviced.

extern "C" 
{
	extern void DSP28x_usDelay(Uint32 Count);

	// --------------------------------------------------- 
	// ISR for PIE INT9.6                                  
	// Connected to eCAN1-INTA  eCAN                       
	// ----------------------------------------------------
//	void eCAN1INT_ISR(void)  // eCAN-A
interrupt void ECAN1INTA_ISR(void) 	{
	   	asm (" NOP");
		
		MIV = ECanaRegs.CANGIF1.bit.MIV1; 
		// Immediate message received.
		if ( MIV == IMMEDIATE_RECEIVE_MAILBOX )
		{
		    Log_info0("ECAN1INTA_ISR / IMMEDIATE_RECEIVE_MAILBOX ");
			// Add Handling code By application here !
			//SEM_post(&SEM_CAN_MSG_RECEIVED);
			Semaphore_post(SEM_CAN_MSG_RECEIVED);

		}
		else if ( MIV == BROADCAST_RECEIVE_MAILBOX )
		{
		    Log_info0("ECAN1INTA_ISR / BROADCAST_RECEIVE_MAILBOX / post SEM_CAN_BROADCAST_MSG_RECEIVED");
			// Application need to create task to handle BROADCAST message.
			// This task will pend on SEM_CAN_BROADCAST_MSG_RECEIVED.
			//SEM_post(&SEM_CAN_BROADCAST_MSG_RECEIVED);
			Semaphore_post(SEM_CAN_BROADCAST_MSG_RECEIVED);

			EALLOW;
			ECanShadow.CANRMP.all = 0;
			ECanShadow.CANRMP.bit.RMP30 = 1;
			ECanaRegs.CANRMP.all = ECanShadow.CANRMP.all;
			EDIS;
		}
		else 
		{
			//SEM_post(&SEM_CAN_MSG_RECEIVED);
		    Log_info1("ECAN1INTA_ISR MIV = %d", MIV);
			Semaphore_post(SEM_CAN_MSG_RECEIVED);
		}
	   	MIV = 0;
   	   	PieCtrlRegs.PIEACK.bit.ACK9 = 1;    // Enables PIE to drive a pulse into the CPU
	   	IER |= 0x0100;					// Enable INT9 
	   	EINT;
	   	return;
	}

	/*  */
	/* ISR for PIE INT9.5 */
	/* Connected to HECC0INTA eCAN */
	/* */
//	void eCAN0INT_ISR(void) // eCAN-A
interrupt void ECAN0INTA_ISR(void)  // eCAN-A
	{
    Log_info0("ECAN1INTA_ISR start ");
		// Clear WDIFO flag bit..
		EALLOW;
		ECanShadow.CANGIF0.all = ECanaRegs.CANGIF0.all;
		ECanShadow.CANGIF0.bit.WDIF0 = 1;
		ECanaRegs.CANGIF0.all = ECanShadow.CANGIF0.all;
		EDIS;
		//int0count++; // Interrupt counter
		
		// Reenable core interrupts and CAN int from PIE module
		PieCtrlRegs.PIEACK.bit.ACK9 = 1; // Enables PIE to drive a pulse into the CPU
		IER |= 0x0100; // Enable INT9
		EINT;
		return;
	}

#if DSP28_2808
	/*  */
	/* ISR for PIE INT9.7      */
	/* Connected to ??? eCAN-B */
	/* */
	void eCAN0INTB_ISR(void)  // eCAN-B
	{
		// Clear WDIFO flag bit..
		EALLOW;
		ECanShadow.CANGIF0.all = ECanbRegs.CANGIF0.all;
		ECanShadow.CANGIF0.bit.WDIF0 = 1;
		ECanbRegs.CANGIF0.all = ECanShadow.CANGIF0.all;
		EDIS;
		//int0count++; // Interrupt counter
		
		// Reenable core interrupts and CAN int from PIE module
		PieCtrlRegs.PIEACK.bit.ACK9 = 1; // Enables PIE to drive a pulse into the CPU
		IER |= 0x0100; // Enable INT9
		EINT;
		return;
	}


	/*  */
	/* ISR for PIE INT9.8      */
	/* Connected to ??? eCAN-B */
	/* */
	void eCAN1INTB_ISR(void)  // eCAN-B
	{
	   	asm (" NOP");
	   	MIV = ECanbRegs.CANGIF1.bit.MIV1; 
		// Immediate message received.
		if ( MIV == IMMEDIATE_RECEIVE_MAILBOX )
		{
			// Add Handling code By application here !
			EALLOW;	 
			ECanShadow.CANRMP.all = 0;
			ECanShadow.CANRMP.bit.RMP31 = 1;
			ECanbRegs.CANRMP.all = ECanShadow.CANRMP.all;
			EDIS;
		}
		else if ( MIV == BROADCAST_RECEIVE_MAILBOX )
		{
			// Application need to create task to handle BROADCAST message.
			// This task will pend on SEM_CAN_BROADCAST_MSG_RECEIVED.
			SEM_post(&SEM_CAN_BROADCAST_MSG_RECEIVED);
			EALLOW;
			ECanShadow.CANRMP.all = 0;
			ECanShadow.CANRMP.bit.RMP30 = 1;
			ECanbRegs.CANRMP.all = ECanShadow.CANRMP.all;
			EDIS;
		}
		else 
		{
			SEM_post(&SEM_CAN_MSG_RECEIVED);
		}
	   	MIV = 0;
   	   	PieCtrlRegs.PIEACK.bit.ACK9 = 1;    // Enables PIE to drive a pulse into the CPU
	   	IER |= 0x0100;					// Enable INT9 
	   	EINT;
	   	return;
	}
#endif // if DSP28_2808

}

/*******************************************************************************
Function Name : CCANPhysicalLayer
Description :  	This is the Constructor function, called from CCANDataLinkLayer Constructor.
*******************************************************************************/
CCANPhysicalLayer::CCANPhysicalLayer()
{
	m_uiFirstMBXfifo = MAILBOX_NUM_2;
	m_uiLastMBXfifo = MAILBOX_NUM_31;
	uiBoard_Id = 0xff;
	m_CanBusSelection = CAN_A_INTERFACE;
}

/*******************************************************************************
Function Name : CCANPhysicalLayer
Description :  	This is the destructor function.
*******************************************************************************/
CCANPhysicalLayer::~CCANPhysicalLayer()
{
}

/*******************************************************************************
Function Name : HWconfig
Description :  	This is the hardware configuration function.
*******************************************************************************/
void CCANPhysicalLayer::HWconfig(CAN_INTERFACE_NUM CanBusSelect)
{
	m_CanBusSelection = CanBusSelect;
#if DSP28_F2812
	InitECan();
#elif DSP28_2808
	if ( m_CanBusSelection == CAN_A_INTERFACE )
	{
		InitECana();
	}
	else if ( m_CanBusSelection == CAN_B_INTERFACE )
	{
		InitECanb();
	}
#else
		InitECana();


#endif
}

/*******************************************************************************
Function Name : ConfigFifoMailboxes
Description :  	This is the hardware configuration function.
*******************************************************************************/
void CCANPhysicalLayer::SetRxFifoMailboxes(Uint16 uiFirstMBXfifo, Uint16 uiLastMBXfifo)
{
	m_uiFirstMBXfifo = uiFirstMBXfifo;
	m_uiLastMBXfifo = uiLastMBXfifo;
}

/*******************************************************************************
Function Name : Init
Description :  	This is the initialization function, 
				set MailBox 0,1 to transmit, set all other MBox to Recive. 
*******************************************************************************/
void CCANPhysicalLayer::Init()
{
    Log_info0("CCANPhysicalLayer::Init ");
	union CAN_HEADER_ID   MbxHeaderId;    

	SetRxFifoMailboxes(MAILBOX_NUM_3,MAILBOX_NUM_29);

	if ( m_CanBusSelection == CAN_A_INTERFACE )
	{
		m_pCanRegs = (ECAN_REGS *)&ECanaRegs;
		m_pCanMBoxes = (ECAN_MBOXES *)&ECanaMboxes;
	}
#if DSP28_2808
	else if ( m_CanBusSelection == CAN_B_INTERFACE )
	{
		m_pCanRegs = (ECAN_REGS *)&ECanbRegs;
		m_pCanMBoxes = (ECAN_MBOXES *)&ECanbMboxes;
	}
#endif

	// set MSB on right 
	EALLOW;	 
	ECanShadow.CANMC.all = (*m_pCanRegs).CANMC.all;
	ECanShadow.CANMC.bit.DBO = 1;      // Data is received or transmitted most significant byte first
	(*m_pCanRegs).CANMC.all = ECanShadow.CANMC.all;
	EDIS;

	// Configure Mailbox 0,1 as transmit
	CanMailboxConfig(TRANSMIT_MAILBOX,MAILBOX_NUM_0, 0x80000000,RESET_OVERWRITE_PROTECT_CONTROL,DISABLE_MAILBOX_INTERRUPT,0);
	CanMailboxConfig(TRANSMIT_MAILBOX,MAILBOX_NUM_1, 0x80000000,RESET_OVERWRITE_PROTECT_CONTROL,DISABLE_MAILBOX_INTERRUPT,0);

	// Configure Charger recieving data mailbox
	CanMailboxConfig(RECEIVE_MAILBOX,CHARGER_RECEIVE_MAILBOX,POWER_SUPPLY_RETURN_VOLTAGE_ID,RESET_OVERWRITE_PROTECT_CONTROL,DISABLE_MAILBOX_INTERRUPT,0);

	memset(&MbxHeaderId,0, sizeof(CAN_HEADER_ID));
	if( uiBoard_Id != 0xff)
	   	MbxHeaderId.bit.BoardID = uiBoard_Id;
	MbxHeaderId.bit.MsgType = CAN_ID_MSG_TYPE_IMMEDIATE;
	CanMailboxConfig(RECEIVE_MAILBOX,IMMEDIATE_RECEIVE_MAILBOX,(Uint32)MbxHeaderId.all/*0x80000004*/,RESET_OVERWRITE_PROTECT_CONTROL,ENABLE_MAILBOX_INTERRUPT, 0xffffffe3);
	MbxHeaderId.bit.MsgType = CAN_ID_MSG_TYPE_BROADCAST;
	CanMailboxConfig(RECEIVE_MAILBOX,BROADCAST_RECEIVE_MAILBOX,(Uint32)MbxHeaderId.all/*0x80000000*/,SET_OVERWRITE_PROTECT_CONTROL,ENABLE_MAILBOX_INTERRUPT, 0xffffffe3);

	// Configure Mailboxe m_uiFirstMBXfifo-m_uiLastMBXfifo for FIFO Receive messages, filtered by board ID.
	for (Uint16 i=m_uiFirstMBXfifo; i <=m_uiLastMBXfifo; i++)
	{
		CanMailboxConfig(RECEIVE_MAILBOX,i,(Uint32)MbxHeaderId.all/*0x80000000*/,SET_OVERWRITE_PROTECT_CONTROL,ENABLE_MAILBOX_INTERRUPT,0xffffe01f);
	}

// Configure CAN interrupts 
//**************************
	EALLOW;	 
	ECanShadow.CANMIL.all = 0xFFFFFFF8 ; // Interrupts asserted on eCAN1INT
//	ECanaShadow.CANMIL.all = 0xFFFFFFFF ; // Interrupts asserted on eCAN1INT
//	ECanaShadow.CANMIL.all  = 0x00000000 ; // Interrupts asserted on eCAN0INT	
	(*m_pCanRegs).CANMIL.all = ECanShadow.CANMIL.all;

/*    ECanaShadow.CANGIM.all = 0;	
    // ECanaShadow.CANGIM.bit.I0EN = 1;   // Enable eCAN1INT or eCAN0INT 
    ECanaShadow.CANGIM.bit.I1EN = 1;
    ECanaRegs.CANGIM.all = ECanaShadow.CANGIM.all;
*/
// Configure the CANGIM register. The flags AAIM, WDIM, WUIM,BOIM, EPIM,
// and WLIM (GIM.14-9) should always be set (enabling these interrupts). 
// In addition, the GIL (GIM.2) bit is set to have the global interrupts on 
// Level 0 (not like the level of the mailbox interrupts. Both the I1EN
// (GIM.1) and I0EN (GIM.0) flags are set to enable both interrupt
// lines. The flag RMLIM (GIM.11) is not set (not to load the CPU)
    ECanShadow.CANGIM.all = 0;    
/*  ECanShadow.CANGIM.bit.AAIM = 1; 	// Abort Acknowledge Interrupt Enabled
    ECanShadow.CANGIM.bit.WDIM = 1; 	// Write denied interrupt Enabled
    ECanShadow.CANGIM.bit.WUIM = 1; 	// Wake-up interrupt Enabled
    ECanShadow.CANGIM.bit.BOIM = 1; 	// Bus-off interrupt Enabled
    ECanShadow.CANGIM.bit.EPIM = 1; 	// Error-passive interrupt Enabled
    ECanShadow.CANGIM.bit.WLIM = 1; 	// Warning level interrupt Enabled
    ECanShadow.CANGIM.bit.RMLIM = 0;   // Received-message-lost interrupt Enabled
    ECanShadow.CANGIM.bit.GIL = 0;  	// All global interrupts are mapped to the ECAN0INT interrupt line (level 0).
    ECanShadow.CANGIM.bit.I0EN = 1;    // Interrupt 0 enable
 */   
 	ECanShadow.CANGIM.bit.I1EN = 1;    // Interrupt 1 enable
    (*m_pCanRegs).CANGIM.all = ECanShadow.CANGIM.all;    
    
    EDIS;   

// Clear all interrupt flag bits 
    (*m_pCanRegs).CANGIF0.all = 0xFFFFFFFF;
    (*m_pCanRegs).CANGIF1.all = 0xFFFFFFFF;
 
/* Reassign ISRs. i.e. reassign the PIE vector for ECAN0INTA_ISR and ECAN0INTA_ISR 
   to point to a different ISR than the shell routine found in DSP28_DefaultIsr.c.
   This is done if the user does not want to use the shell ISR routine but instead
   wants to embed the ISR in this file itself. */
	
	// Configure PIE interrupts 
#if DSP28_F2812
  	PieCtrlRegs.PIECRTL.bit.ENPIE = 1;  // Enable vector fetching from PIE block	
#elif DSP28_2808
	PieCtrlRegs.PIECTRL.bit.ENPIE = 1;
#else
	PieCtrlRegs.PIECTRL.bit.ENPIE = 1;

#endif

	PieCtrlRegs.PIEACK.bit.ACK9 = 1;    // Enables PIE to drive a pulse into the CPU

// The interrupt can be asserted in either of the eCAN interrupt lines
#if DSP28_F2812
//	PieCtrlRegs.PIEIER9.bit.INTx5 = 1;  // Enable INTx.5 of INT9 (eCAN0INT)
	PieCtrlRegs.PIEIER9.bit.INTx6 = 1;  // Enable INTx.6 of INT9 (eCAN1INT)
#elif DSP28_2808
	if ( m_CanBusSelection == CAN_A_INTERFACE )
	{
	//	PieCtrlRegs.PIEIER9.bit.INTx5 = 1;  // Enable INTx.5 of INT9 (eCAN0INT)
		PieCtrlRegs.PIEIER9.bit.INTx6 = 1;  // Enable INTx.6 of INT9 (eCAN1INT)
	}
	else if ( m_CanBusSelection == CAN_B_INTERFACE )
	{
		//PieCtrlRegs.PIEIER9.bit.INTx7 = 1;  // Enable INTx.7 of INT9 (eCANB0INT)
		PieCtrlRegs.PIEIER9.bit.INTx8 = 1;  // Enable INTx.8 of INT9 (eCANB1INT)
	}
#else
	if ( m_CanBusSelection == CAN_A_INTERFACE )
	{
	//	PieCtrlRegs.PIEIER9.bit.INTx5 = 1;  // Enable INTx.5 of INT9 (eCAN0INT)
		PieCtrlRegs.PIEIER9.bit.INTx6 = 1;  // Enable INTx.6 of INT9 (eCAN1INT)
	}
	else if ( m_CanBusSelection == CAN_B_INTERFACE )
	{
		//PieCtrlRegs.PIEIER9.bit.INTx7 = 1;  // Enable INTx.7 of INT9 (eCANB0INT)
		PieCtrlRegs.PIEIER9.bit.INTx8 = 1;  // Enable INTx.8 of INT9 (eCANB1INT)
	}

#endif
	
	// Configure system interrupts
	IER |= 0x0100;		
	EINT; 	// Global enable of interrupts  

}

/*******************************************************************************
Function Name : TransmitMsg
Description :  	This is the Transmit function.
*******************************************************************************/
STATUS CCANPhysicalLayer::TransmitMsg(CAN_MESSAGE * pCan_Bus_Data,Uint16 uiTxPriority, Uint32 nCountToWait)
{
    Log_info0("TransmitMsg start ");
	struct MBOX tempMB;
	Uint32 nCount;// = nCountToWait;
	Uint16 uiMailBoxNumber;

	if (pCan_Bus_Data == NULL)
	  return 0;

	if( uiTxPriority == CAN_ID_PRIORITY_LEVEL_NORMAL )
	{
		uiMailBoxNumber = MAILBOX_NUM_0;
	}
	else if( uiTxPriority == CAN_ID_PRIORITY_LEVEL_HIGH )
	{
		uiMailBoxNumber = MAILBOX_NUM_1;
	}

	tempMB.MDL.all = pCan_Bus_Data->uiPacketDataL;
	tempMB.MDH.all = pCan_Bus_Data->uiPacketDataH;
	tempMB.MSGID.all = pCan_Bus_Data->MsgId.all;
	tempMB.MSGID.bit.IDE = 1; // The transmit message had an extended identifier(AMI = 1, IDE = 1)
	tempMB.MSGCTRL.all =0;
	tempMB.MSGCTRL.bit.DLC = pCan_Bus_Data->uiPacketDataArrSize;
	ECanShadow.CANME.all = (*m_pCanRegs).CANME.all;
	
	switch (uiMailBoxNumber)
	{
		case MAILBOX_NUM_0:		
				ECanShadow.CANME.bit.ME0 = 0;  // Disable  Mailbox
				(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;
				(*m_pCanMBoxes).MBOX0.MSGID.all = tempMB.MSGID.all;
				(*m_pCanMBoxes).MBOX0.MSGCTRL.all = 0;
				(*m_pCanMBoxes).MBOX0.MSGCTRL.all = tempMB.MSGCTRL.all;
				(*m_pCanMBoxes).MBOX0.MDL.all = tempMB.MDL.all;
				(*m_pCanMBoxes).MBOX0.MDH.all = tempMB.MDH.all;
				ECanShadow.CANME.bit.ME0 = 1;  // Enable  Mailbox
				(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;

				ECanShadow.CANTRS.all=(*m_pCanRegs).CANTRS.all;
				ECanShadow.CANTRS.bit.TRS0=1;
				(*m_pCanRegs).CANTRS.all=ECanShadow.CANTRS.all;
				for (nCount =0; nCount < nCountToWait; nCount++)
				{
					if((*m_pCanRegs).CANTA.bit.TA0==1) // wait for end of transmission
						break;
					else 
					{
						// wait 100 micro for end of transmission
						DELAY_US(100);
					}
				}
				ECanShadow.CANTA.all=0;
				ECanShadow.CANTA.bit.TA0=1;
				(*m_pCanRegs).CANTA.all=ECanShadow.CANTA.all;	// Reset transmission
				break;
		case MAILBOX_NUM_1:
				ECanShadow.CANME.bit.ME1 = 0;  // Disable  Mailbox
				(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;
				(*m_pCanMBoxes).MBOX1.MSGID.all = tempMB.MSGID.all;
				(*m_pCanMBoxes).MBOX1.MSGCTRL.all = 0;
				(*m_pCanMBoxes).MBOX1.MSGCTRL.all = tempMB.MSGCTRL.all;
				(*m_pCanMBoxes).MBOX1.MDL.all = tempMB.MDL.all;
				(*m_pCanMBoxes).MBOX0.MDH.all = tempMB.MDH.all;
				ECanShadow.CANME.bit.ME1 = 1;  // Enable  Mailbox
				(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;

				ECanShadow.CANTRS.all=(*m_pCanRegs).CANTRS.all;
				ECanShadow.CANTRS.bit.TRS1=1;
				(*m_pCanRegs).CANTRS.all=ECanShadow.CANTRS.all;
				// Check if mailbox transmission ends
				for (nCount =0; nCount < nCountToWait; nCount++)
				{
					if((*m_pCanRegs).CANTA.bit.TA1==1) // wait for end of transmission
						break;
					else 
					{
						// wait 100 micro for end of transmission
						DELAY_US(100);
					}
				}
				ECanShadow.CANTA.all=0;
				ECanShadow.CANTA.bit.TA1=1;
				(*m_pCanRegs).CANTA.all=ECanShadow.CANTA.all;	// Reset transmission
				break;
		default:
				return CAN_INVALID_TRANSMIT_MAILBOX_ERROR;
	}
	if( nCount != nCountToWait ) 
		return NO_ERROR;
	else
		return  CAN_FAIL_TO_TRANSMIT_ERROR;
}

/*******************************************************************************
Function Name : Receive
Description :  	This is the Receive function.
*******************************************************************************/
STATUS CCANPhysicalLayer::ReceiveFifoMsg(CAN_MESSAGE * pCan_Bus_Data)
{
    Log_info0("ReceiveFifoMsg start ");
	static Uint16 uiMbxNum;
	Uint32 uiMBMask = 0x01;
	Uint32 uiRMPMask = 0xffffffff;
   	volatile struct MBOX *Mailbox;


	// Check first if IMMEDIATE_RECEIVE_MAILBOX
	if( (*m_pCanRegs).CANRMP.bit.RMP31 != 0 )
	{
	    Log_info0("ReceiveFifoMsg /IMMEDIATE_RECEIVE_MAILBOX ");
		pCan_Bus_Data->MsgId.all = ECanaMboxes.MBOX31.MSGID.all;
		EALLOW;	 
		ECanShadow.CANRMP.all = 0;
		ECanShadow.CANRMP.bit.RMP31 = 1;
		ECanaRegs.CANRMP.all = ECanShadow.CANRMP.all;
		EDIS;

		return MAILBOX_NUM_31;
		//CanRxData.MsgId.bit.VirtualRegID = sMsgId.bit.VirtualRegID;
		//CanRxData.MsgId.bit.BoardID = sMsgId.bit.BoardID;
		//uiSSid = (sMsgId.bit.FCopmSSID_L)+(sMsgId.bit.FCopmSSID_H<<3);
	}

	// Start check from high priority mailbox.
	uiMbxNum = m_uiLastMBXfifo;
	// Check only the FIFO receive mailboxes !
	uiRMPMask &= uiRMPMask<<m_uiFirstMBXfifo;
	uiRMPMask &= uiRMPMask>>(MAILBOX_NUM_31-m_uiLastMBXfifo);
	while( ((*m_pCanRegs).CANRMP.all & uiRMPMask) != 0 )
	{
		uiMBMask = uiMBMask<<uiMbxNum;
		if (((*m_pCanRegs).CANRMP.all & uiMBMask))
		{
			// reset the MB first
			(*m_pCanRegs).CANRMP.all |= uiMBMask;
			Mailbox = &m_pCanMBoxes->MBOX0 + uiMbxNum; //&ECanaMboxes.MBOX0 + uiMbxNum;
			pCan_Bus_Data->uiPacketDataL = Mailbox->MDL.all; 
			pCan_Bus_Data->uiPacketDataH = Mailbox->MDH.all; 
			pCan_Bus_Data->uiPacketDataArrSize = Mailbox->MSGCTRL.bit.DLC;
			pCan_Bus_Data->MsgId.all = (Uint32)Mailbox->MSGID.all;
		    Log_info1("ReceiveFifoMsg /uiMbxNum=%d",uiMbxNum );
			return uiMbxNum;
		}
		else 
		{
			uiMBMask = 0x01;
			if( uiMbxNum > m_uiFirstMBXfifo)
			{
				uiMbxNum--;
			}
			else
			{
				uiMbxNum = m_uiLastMBXfifo; 
			}
		}
	}

    Log_info0("ReceiveFifoMsg /CAN_NO_MESSAGE_WAIT_ERROR" );
	return CAN_NO_MESSAGE_WAIT_ERROR;
}


/*******************************************************************************
Function Name : Receive
Description :  	This is the Receive function.
*******************************************************************************/
STATUS CCANPhysicalLayer::ReceiveSpecialMbx(CAN_MESSAGE * pCan_Bus_Data, Uint16 MailBox_Serial_Number)
{

    Log_info0("ReceiveSpecialMbx start ");
	Uint32 uiMBMask = 0x01;
   	volatile struct MBOX *Mailbox;
	Uint16 uiRet = NO_ERROR;

	uiMBMask = uiMBMask<<MailBox_Serial_Number;
	if (((*m_pCanRegs).CANRMP.all & uiMBMask))
	{
		// reset the MB first
		(*m_pCanRegs).CANRMP.all |= uiMBMask;
		Mailbox =  &m_pCanMBoxes->MBOX0 + MailBox_Serial_Number;// &ECanaMboxes.MBOX0 + MailBox_Serial_Number;
		pCan_Bus_Data->uiPacketDataL = Mailbox->MDL.all; 
		pCan_Bus_Data->uiPacketDataH = Mailbox->MDH.all; 
		pCan_Bus_Data->uiPacketDataArrSize = Mailbox->MSGCTRL.bit.DLC;
		pCan_Bus_Data->MsgId.all = (Uint32)Mailbox->MSGID.all;
		uiRet = NO_ERROR;
	}
	else
	{
		uiRet = CAN_NO_MESSAGE_WAIT_ERROR;
	}

	return uiRet;
}

void CCANPhysicalLayer::CanMailboxConfig(Bool bDir, Uint16 uiMBNunmer, Uint32 uiMSGID,Bool bOPC,Bool bEnableInt, Uint32 uiMaskID)
{

	if( uiMBNunmer > MAILBOX_NUM_31 )
	{
		   Log_info0("CanMailboxConfig end uiMBNunmer > MAILBOX_NUM_31");
		return;	
	}
	
	volatile Uint32 *pMSGIDReg;
	volatile Uint32 *pLAMReg;

	if ( m_CanBusSelection == CAN_A_INTERFACE )
	{
		pMSGIDReg = (&ECanaMboxes.MBOX0.MSGID.all);
		pLAMReg = (&ECanaLAMRegs.LAM0.all);
	}

#if DSP28_2808
	else if ( m_CanBusSelection == CAN_B_INTERFACE )
	{
		pMSGIDReg = (&ECanbMboxes.MBOX0.MSGID.all);
		pLAMReg = (&ECanbLAMRegs.LAM0.all);
	}
#endif
	// Disable Mailboxes 
	ECanShadow.CANME.all = (*m_pCanRegs).CANME.all;
	ECanShadow.CANME.all = 0x0;  // Disable all Mailbox
	(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;

	// set Recive/transmit Mailbox Direction
	ECanShadow.CANMD.all = (*m_pCanRegs).CANMD.all;	
	ECanShadow.CANMD.all |= (Uint32)bDir<<(Uint32)uiMBNunmer;
	(*m_pCanRegs).CANMD.all = ECanShadow.CANMD.all;	

	// Check if acceptance mask is used.
	if ( uiMaskID != 0 )
	{
		// AME = 1 - The corresponding acceptance mask is used.
		uiMSGID|=0x40000000;
	}
	// The RECEIVED message had an extended identifier(AMI = 1, IDE = 1)
	// Set the uiMSGID in the requested mailbox.
	(*(pMSGIDReg+(uiMBNunmer*4))) = uiMSGID|0x80000000;

	// filtering (1 = dont care) some identifier bits of the incoming message.
	// Set uiMaskID	in the requested mailbox.
	(*(pLAMReg+uiMBNunmer)) = uiMaskID;

	// Set bOPC in the requested mailbox.
	ECanShadow.CANOPC.all = (*m_pCanRegs).CANOPC.all;	
	ECanShadow.CANOPC.all |= (Uint32)bOPC<<(Uint32)uiMBNunmer;
	(*m_pCanRegs).CANOPC.all = ECanShadow.CANOPC.all;	

	EALLOW;	 
	// Set bEnableInt in the requested mailbox.
	ECanShadow.CANMIM.all = (*m_pCanRegs).CANMIM.all;	
	ECanShadow.CANMIM.all |= (Uint32)bEnableInt<<(Uint32)uiMBNunmer;
	(*m_pCanRegs).CANMIM.all = ECanShadow.CANMIM.all;	

	// Enable Mailboxes 
	ECanShadow.CANME.all = (*m_pCanRegs).CANME.all;
	ECanShadow.CANME.all = 0xFFFFFFFF;  // enable all Mailbox
//	ECanaShadow.CANME.all |= (Uint32)0x01<<(Uint32)uiMBNunmer;
	(*m_pCanRegs).CANME.all = ECanShadow.CANME.all;

	EDIS;
}





