//###########################################################################
//
// FILE:   HO20Wcommands.h
//
// Description: Include file for the MRIPL communication protocol commands.
//
//###########################################################################
#ifndef HO20W_COMMANDS_H
#define HO20W_COMMANDS_H

//---------------------------------------------------------------------------
#include "DSP281x_GlobalPrototypes.h"  // Prototypes for global functions within the 					// Include DSP/BIOS Header Files
#include "DSP281x_Device.h"			   // Peripheral address definitions
#include "std.h"


#define CMD_ERROR_REPORT 0xF2
//Err handler messages (sent to board sub system)
#define CMD_ERROR_HANDLER_ERR_SET	0xF3
#define CMD_ERROR_HANDLER_CLEAR_ALL	0xF4
#define CMD_INFORM_RFID_STATUS      0xF5
#define CMD_SET_LIMP_MODE           0xF6
#define CMD_UPDATE_SELF_TEST_STATE  0xF7
#define CMD_UPDATE_REQ_ENTER_STATE  0xF8
#define CMD_SEND_DEBUG_STRING       0xFE
//Log File messages (sent to board sub system)
#define CMD_ERROR_HANDLER_READ_ERROR_RECORD_NUM		0xF9
#define CMD_ERROR_HANDLER_RECEIVE_ERROR_RECORD_NUM	0xFA
#define CMD_ERROR_HANDLER_READ_ERROR_RECORD			0xFB
#define CMD_ERROR_HANDLER_RECEIVE_ERROR_RECORD		0xFC
#define CMD_ERROR_HANDLER_CLEAR_LOG_FILE			0xFD


// Sub Systems Types
enum {
		SUBSYSTEM_TYPE_BLUEBOX =1,
		SUBSYSTEM_TYPE_CHARGER = 2,
		SUBSYSTEM_TYPE_COOLING = 3,
		SUBSYSTEM_TYPE_SWITCHING =4,
		SUBSYSTEM_TYPE_EXT_INDICATIONS =5,
		SUBSYSTEM_TYPE_LASING_CNT = 6,
		SUBSYSTEM_TYPE_BOARD =7
};

// Service modality commands
enum {
		SERVICE_MOD_CMD_ENTER_READY =1,
		SERVICE_MOD_CMD_ENTER_STANDBY =2,
		SERVICE_MOD_CMD_SYSTEM_ON =3,
		SERVICE_MOD_CMD_SYSTEM_OFF =4
};

// Sub System Opcodes
enum {
		SUBSYSTEM_CMD_BLUEBOX = 1,
		SUBSYSTEM_CMD_CHARGER = 2,
		SUBSYSTEM_CMD_COOLING = 3,
		SUBSYSTEM_CMD_SWITCHING = 4,
		SUBSYSTEM_CMD_E_INDICATIONS = 5,
		SUBSYSTEM_CMD_LASING_CNT = 6,
		SUBSYSTEM_CMD_BOARD = 7	,
		CALIBRATION_CMD=8,
		SIS_CMD=9
};

// Blue Box Sub System Commands Opcodes
enum {

		BLUEBOX_CMD_CREATE_FC = 100,
		BLUEBOX_CMD_GET_FC_STATUS = 101,
		BLUEBOX_CMD_DESTROY_FC = 102,
		BLUEBOX_CMD_GET_ALL_FC_STATUS = 103,
		BLUEBOX_CMD_OPERATE = 104,
		BLUEBOX_CMD_RESET = 110,
		BLUEBOX_CMD_ERROR_REPORT = CMD_ERROR_REPORT
};

// Functional Components Opcodes for BB operate functions
enum {
		BLUEBOX_CMD_GPIO_HWCONFIG = 1,
		BLUEBOX_CMD_GPIO_INIT = 2,
		BLUEBOX_CMD_GPIO_SET_ON = 3,
		BLUEBOX_CMD_GPIO_SET_OFF = 4,
		BLUEBOX_CMD_GPIO_IS_ON = 5,
		BLUEBOX_CMD_GPIO_IS_OFF = 6,
		BLUEBOX_CMD_GPIO_TOGGLE = 7,

		BLUEBOX_CMD_I2C_GPIO_HWCONFIG =0x11,
		BLUEBOX_CMD_ANALOG_INPUT_HWCONFIG = 0x21,

		// I2C ADC opcods.
		BLUEBOX_CMD_I2C_ANALOG_INPUT_HWCONFIG = 0x31,
		BLUEBOX_CMD_I2C_ANALOG_INPUT_REQ_VALUE = 0x32,
		BLUEBOX_CMD_I2C_ANALOG_INPUT_RES_VALUE = 0x33,

		// Spi EEprom opcods.
		BLUEBOX_CMD_SPI_EE_HWCONFIG = 0x41,
		BLUEBOX_CMD_SPI_EE_WRITE_BYTE = 0x42,
		BLUEBOX_CMD_SPI_EE_READ_BYTE_REQ = 0x43, 	// Read byte request
		BLUEBOX_CMD_SPI_EE_WRITE_WORD = 0x44,
		BLUEBOX_CMD_SPI_EE_READ_WORD_REQ = 0x45, 	// Read word request
		BLUEBOX_CMD_SPI_EE_WRITE_BUFFER = 0x46,
		BLUEBOX_CMD_SPI_EE_READ_BUFFER_REQ = 0x47,	// Read buffer request
		BLUEBOX_CMD_SPI_EE_WRITE_PROTECT = 0x48,
		BLUEBOX_CMD_SPI_EE_READ_BYTE_RES = 0x49,	// Read byte response
		BLUEBOX_CMD_SPI_EE_READ_WORD_RES = 0x4A,	// Read word response
		BLUEBOX_CMD_SPI_EE_READ_BUFFER_RES = 0x4B,	// Read buffer response

		// One Wire EEprom opcods.
		BLUEBOX_CMD_ONEWIRE_EE_HWCONFIG = 0x51,
		BLUEBOX_CMD_ONEWIRE_EE_WRITE = 0x52,
		BLUEBOX_CMD_ONEWIRE_EE_READ_REQ = 0x53,
		BLUEBOX_CMD_ONEWIRE_EE_READ_RES = 0x54,
		BLUEBOX_CMD_ONEWIRE_EE_SEARCH_REQ = 0x55,
		BLUEBOX_CMD_ONEWIRE_EE_SEARCH_RES = 0x56,
		BLUEBOX_CMD_ONEWIRE_SET_DEVICE_ID = 0x57,

		BLUEBOX_CMD_I2C_EE_HWCONFIG = 0x61,
		BLUEBOX_CMD_PULSEGEN_CURRENT_CONTROL_HWCONFIG =0x71,

		BLUEBOX_CMD_PULSEGEN_ENVELOPE_HWCONFIG = 0x81 ,
		BLUEBOX_CMD_PULSEGEN_ENVELOPE_SET_PARAMS = 0x82,
		BLUEBOX_CMD_PULSEGEN_ENVELOPE_FIRE_PULSE =0x83,

		BLUEBOX_CMD_PWM_ANALOG_OUTPUT_HWCONFIG = 0x91 ,

		BLUEBOX_CMD_SPI_INTERFACE_HWCONFIG = 0xA1,
		BLUEBOX_CMD_SPI_INTERFACE_SET_CHANNEL = 0xA2,
		BLUEBOX_CMD_SPI_INTERFACE_TRANSMIT = 0xA3,
		BLUEBOX_CMD_SPI_INTERFACE_RECIVE = 0xA4,

		BLUEBOX_CMD_I2C_INTERFACE_HWCONFIG = 0xB1,
		BLUEBOX_CMD_I2C_INTERFACE_WRITE = 0xB2,
		BLUEBOX_CMD_I2C_INTERFACE_READ = 0xB3,
		BLUEBOX_CMD_I2C_INTERFACE_READ_AFTER_WRITE = 0xB4,

		BLUEBOX_CMD_FPGA_INTERFACE_HWCONFIG = 0xC1,
		BLUEBOX_CMD_FPGA_INTERFACE_WRITE = 0xC2,
		BLUEBOX_CMD_FPGA_INTERFACE_READ = 0xC3,
		BLUEBOX_CMD_FPGA_INTERFACE_READ_AFTER_WRITE = 0xC4,

		BLUEBOX_CMD_ANALOG_OUT_SPI_HWCONFIG = 0xD1,
		BLUEBOX_CMD_ANALOG_OUT_SPI_SET_VALUE = 0xD2,

		BLUEBOX_CMD_I2C_PORT_HWCONFIG = 0xE1,
		BLUEBOX_CMD_I2C_PORT_SET_VALUE = 0xE2,
		BLUEBOX_CMD_I2C_PORT_GET_VALUE = 0xE3


};



// Cooling SubSystem commands
enum{
	COOLING_CMD_REQUEST_ALL_STATUS = 1,
	COOLING_CMD_RECIVE_ALL_STATUS,
	COOLING_CMD_SET_FAN_ON,
	COOLING_CMD_SET_FAN_OFF,
	COOLING_CMD_SET_PUMP_ON,
	COOLING_CMD_SET_PUMP_OFF,
	COOLING_CMD_SET_COOLING_ON,
	COOLING_CMD_SET_COOLING_OFF,
	COOLING_CMD_ENTER_STATE,
	COOLING_CMD_REQUEST_WATER_TEMP,
	COOLING_CMD_RECIVE_WATER_TEMP,
	COOLING_CMD_REQUEST_FLOW_SWITCH_STATUS,
	COOLING_CMD_RECIVE_FLOW_SWITCH_STATUS,
	COOLING_CMD_REQUEST_THERMAL_SWITCH_STATUS,
	COOLING_CMD_RECIVE_THERMAL_SWITCH_STATUS,
	COOLING_CMD_REQUEST_COOLING_24V_STATUS,
	COOLING_CMD_RECEIVE_COOLING_24V_STATUS,
	COOLING_CMD_SET_INPUTS_SIMULATION_REG,
	COOLING_CMD_ERROR_REPORT
};

enum{
	SWITCHING_CMD_REQUEST_ALL_STATUS = 1,
	SWITCHING_CMD_RECIVE_ALL_STATUS,
	SWITCHING_CMD_SET_ENABLE,
	SWITCHING_CMD_ENTER_STATE,
	SWITCHING_CMD_REQUEST_SIMMER_STATUS,
	SWITCHING_CMD_RECEIVE_SIMMER_STATUS,
	SWITCHING_CMD_REQUEST_IGBT_LEACKAGE_STATUS,
	SWITCHING_CMD_RECEIVE_IGBT_LEACKAGE_STATUS,
	SWITCHING_CMD_REQUEST_OVER_TEMP_STATUS,
	SWITCHING_CMD_RECEIVE_OVER_TEMP_STATUS,
	SWITCHING_CMD_REQUEST_SWITCHING_24V_STATUS,
	SWITCHING_CMD_RECEIVE_SWITCHING_24V_STATUS,
	SWITCHING_CMD_SET_INPUTS_SIMULATION_REG,
	SWITCHING_CMD_ERROR_REPORT
};

enum{
	LASING_CMD_REQUEST_ALL_STATUS = 1,
	LASING_CMD_RECIVE_ALL_STATUS,
	LASING_CMD_ENTER_STATE,
	LASING_CMD_SYSTEM_ENTER_STATE,
    LASING_CMD_SYSTEM_STATE_ENTERED,
	LASING_CMD_SYSTEM_GET_SYS_STATE,
	LASING_CMD_SET_USER_TP,//rate index,energy index
	LASING_CMD_SET_SERVICE_TP,//rate index,pumped energy in J
   	LASING_CMD_SET_LASING_MODE,//service, user,..
	LASING_CMD_SET_FIRE_TYPE,//counted+num of pulses,continius
    LASING_CMD_SET_AIM_BEAM_INDEX,//For leve determine
	LASING_CMD_SET_AIM_BEAM_MODE,//blink,off,on
	LASING_SET_SHUTTER_MODE,//open, close, normal
	LASING_CLEAR_TOTAL_NUM_OF_PULSES,
	LASING_SYSTEM_FAIL_ENTER_STATE,
	LASING_CMD_SET_INPUTS_SIMULATION_REG1,
	LASING_CMD_SET_INPUTS_SIMULATION_REG2,
	LASING_CMD_PERFORM_CALIBRATION,
	LASING_CMD_REQUEST_LASER_CALIBRATION_RESULT,
	LASING_CMD_RECIVE_LASER_CALIBRATION_RESULT,
	LASING_CMD_CALIBRATION_STARTED,
	LASING_CMD_CALIBRATION_FINISHED,
	LASING_CMD_REQUEST_CALIBRATION_IN_PROCESS_STATUS,
	LASING_CMD_RECEIVE_CALIBRATION_IN_PROCESS_STATUS,
	LASING_CMD_CLEAR_NON_PEMANENT_ERRORS,
	LASING_CMD_ACTIVATE_BURN_IN,
	LASING_CMD_SET_BURNIN_TP,
	LASING_CMD_RESUME_BURNIN_FIRE,
	LASING_CMD_READ_PYROS_RECORD_DATA_LOG_FILE,
	LASING_CMD_RECEIVE_PYROS_RECORD_DATA_LOG_FILE,
	LASING_CMD_READ_PYROS_DATA_LOG_FILE,
	LASING_CMD_RECEIVE_PYROS_DATA_LOG_FILE,
	LASING_CMD_PYROS_DATA_CLEAR_LOG_FILE,
    LASING_CMD_REQ_AIM_BEAM_INDEX_MODE,
	LASING_CMD_RECEIVE_AIM_BEAM_INDEX_MODE,
	LASING_CMD_SIMUL_REQUEST_SIS_READ,
	LASING_CMD_SIMUL_REQUEST_SIS_WRITE,
	LASING_CMD_SET_PULSE_WIDTH_SWM,
	LASING_CMD_ERROR_REPORT,
	LASING_CMD_SET_DC_AIM_BEAM_LEVEL,
	LASING_CMD_SAVE_AIM_BEAM_LEVELS,
	LASING_CMD_GET_AIM_BEAM_LEVELS,
	LASING_CMD_RECEIVE_AIM_BEAM_LEVELS
};

enum{
	EX_INDICATIONS_CMD_REQUEST_ALL_STATUS = 1,
	EX_INDICATIONS_CMD_RECIVE_ALL_STATUS,
	EX_INDICATIONS_CMD_SET_FSW_ENABLE,
	EX_INDICATIONS_CMD_ENTER_STATE,
	EX_INDICATIONS_CMD_REQUEST_INTERLOCK_STATE,
	EX_INDICATIONS_CMD_RECIVE_INTERLOCK_STATE,
	EX_INDICATIONS_CMD_SET_LASER_LAMP_STATE,
	EX_INDICATIONS_CMD_SET_BUZZER_ENABLE,
	EX_INDICATIONS_CMD_SET_BUZZER_VOLUME,
	EX_INDICATIONS_CMD_SET_BUZZER_SOUND_TYPE,
	EX_INDICATIONS_CMD_PERFORM_FSW_TEST,
	EX_INDICATIONS_CMD_SET_INPUTS_SIMULATION_REG,
	EX_INDICATIONS_CMD_SET_FSW_PRESSED_SIMULATION,
	EX_INDICATIONS_CMD_ERROR_REPORT
};


enum{
	CHARGER_CMD_REQUEST_ALL_STATUS = 1,
	CHARGER_CMD_RECIVE_ALL_STATUS,
	CHARGER_CMD_SET_CHARGE_ON,
	CHARGER_CMD_SET_DUMP_ON,
	CHARGER_CMD_ENTER_STATE,
	CHARGER_CMD_REQUEST_CHARGER_DONE_STATE,
	CHARGER_CMD_RECIVE_CHARGER_DONE_STATE,
	CHARGER_CMD_REQUEST_CHARGER_DUMP_STATE,
	CHARGER_CMD_RECIVE_CHARGER_DUMP_STATE,
	CHARGER_CMD_REQUEST_CAP_VOLTAGE,//400V-850V
	CHARGER_CMD_RECIVE_CAP_VOLTAGE,
	CHARGER_CMD_REQUEST_P5_LV,
	CHARGER_CMD_RECIVE_P5_LV,
	CHARGER_CMD_REQUEST_P15_LV,
	CHARGER_CMD_RECIVE_P15_LV,
	CHARGER_CMD_REQUEST_M15_LV,
	CHARGER_CMD_RECIVE_M15_LV,
	CHARGER_CMD_REQUEST_P24_LV,
	CHARGER_CMD_RECIVE_P24_LV,
	CHARGER_CMD_REQUEST_CHARGER_FAIL_STATUS,
	CHARGER_CMD_RECIVE_CHARGER_FAIL_STATUS,
	CHARGER_CMD_REQUEST_CHARGER_LV_FAIL_STATUS,
	CHARGER_CMD_RECIVE_CHARGER_LV_FAIL_STATUS,
	CHARGER_CMD_REQUEST_CAP_BALANCE_FAIL_STATUS,
	CHARGER_CMD_RECIVE_CAP_BALANCE_FAIL_STATUS,
	CHARGER_CMD_SET_INPUTS_SIMULATION_REG_1,
	CHARGER_CMD_SET_INPUTS_SIMULATION_REG_2,
	CHARGER_CMD_ERROR_REPORT
};

enum{
	BOARD_CMD_REQUEST_SOFTWARE_VERSION=1,
	BOARD_CMD_RECIVE_SOFTWARE_VERSION,
	BOARD_CMD_REQUEST_FPGA_DIAGNOSTIC,
	BOARD_CMD_RECIVE_FPGA_DIAGNOSTIC,
	BOARD_CMD_START_PERIOD_CHECK,
	BOARD_CMD_STOP_PERIOD_CHECK,
	BOARD_CMD_REQUEST_MACHINE_INFO,
	BOARD_CMD_RECIVE_MACHINE_INFO,
	BOARD_CMD_REQUEST_EE_WRITE_MACHINE_INFO,
	BOARD_CMD_WRITE_TOTAL_WORKING_HOURS,
	BOARD_CMD_REQUEST_DATE_TIME,
	BOARD_CMD_RECEIVED_DATE_TIME,
	BOARD_CMD_SET_DATE_TIME,
	BOARD_SAVE_A_TO_D_CALIBRATION_VALUES,
    BOARD_GET_A_TO_D_CALIBRATION_VALUES,
    BOARD_RECEIVE_A_TO_D_CALIBRATION_VALUES,
	BOARD_GET_FPGA_CONTROLLER_VERSION,
    BOARD_RECEIVE_FPGA_CONTROLLER_VERSION,
	BOARD_GET_SIS_DATA_FOR_SERVICE,
    BOARD_RECEIVE_SIS_DATA_FOR_SERVICE_1,
	BOARD_RECEIVE_SIS_DATA_FOR_SERVICE_2,
	BOARD_RECEIVE_SIS_DATA_FOR_SERVICE_3,
	BOARD_RECEIVE_SIS_DATA_FOR_SERVICE_4,
	BOARD_START_SCREEN_CALIBRATION,
	BOARD_START_SPLASH_SCREEN_BURNING,
	BOARD_CMD_REQUEST_ALL_STATUS = 0xF0,
	BOARD_CMD_RECIVE_ALL_STATUS = 0xF1,
	BOARD_CMD_ERROR_REPORT = CMD_ERROR_REPORT


};


enum
{
	PERIOD_CHECK_HEAD_SUBSYSTEM=0x0001,
	PERIOD_CHECK_SWM_SUBSYSTEM=0x0002,
	PERIOD_CHECK_COOLING_SUBSYSTEM=0x0004,
	PERIOD_CHECK_POWER_METER_SUBSYSTEM=0x0008,
	PERIOD_CHECK_CHARGER_SUBSYSTEM=0x0010,
	PERIOD_CHECK_EX_INDICATIONS_SUBSYSTEM=0x0011,

	PERIOD_CHECK_WITH_I2C=0x8000,
	PERIOD_CHECK_ALL_SUBSYSTEMS_WITHOUT_I2C=0x7fff,
	PERIOD_CHECK_ALL_SUBSYSTEMS_WITH_I2C=0xffff
};

//---------------------------------------------------------------------------
#endif  // end of MRIPL_COMMANDS_H definition

/*** end of file *****************************************************/
